@@ -56,6 +56,8 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>		/* for inet_ntoa() */
 
+#include "sizeof.h"
+
 #if !defined(MAXHOSTNAMELEN) || (MAXHOSTNAMELEN < 64)
 #undef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 256
@@ -265,7 +267,7 @@ static bool user_match (const char *tok, const char *string)
 static const char *resolve_hostname (const char *string)
 {
 	int              gai_err;
-	char             *addr_str;
+	const char       *addr_str;
 	struct addrinfo  *addrs;
 
 	static char      host[MAXHOSTNAMELEN];
